==> pan-0.6.1.tar.gz: pan-0.6.1/pan/db.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 TODO:
 [ ] transparent (de)compression of database (bzip2 preferably)
*/

#include <gnome.h>
#include <gdbm.h>
#include <errno.h>

#include "pan.h"


int
db_group_index_erase (group_data *gdata)
{
	char path[PATH_MAX];

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);
	if (unlink (path) < 0)
		return -1;
	else
		return 1;
}


void
db_erase_article (article_data *adata)
{
	datum key;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_return_if_fail(adata);

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, adata->group->name);

	if ((dbf = gdbm_open(path, 0, GDBM_WRITER | GDBM_FAST, 0600, 0)) == NULL) {
		g_warning(_("We couldn't open the article database:\n%s\n"), path);
		return;
	}

	key.dptr = adata->message_id;
	key.dsize = strlen(adata->message_id) + 1;

	if (gdbm_delete(dbf, key) == -1)
		g_warning("article_db_erase(%s) failed\n", adata->message_id);
	gdbm_close(dbf);
}


int
group_index_exists (group_data *gdata)
{
	char path[PATH_MAX];

	g_snprintf(path, 256, "/%s/%s/%s.gdbm", data_dir,
		   Pan.active_server->name, gdata->name);

	if (g_file_exists(path))
		return TRUE;
	else
		return FALSE;
}


int
save_article_gslist (GSList *p, GDBM_FILE dbf, int count, int total)
{
	article_data *adata;
	time_t current_time;
	datum key, content;
	char *data = NULL;

	time(&current_time);

	for (; p; p = p->next) {
		adata = p->data;
		data = g_strdup_printf ("%d\n%d\n%d\n%d\n%d\n%s\n%s\n%s\n%s",
					(int) adata->date, adata->state & ~STATE_NEW,
					adata->linecount, adata->parts, adata->part,
					adata->number, adata->subject,
					adata->author, adata->references);

		content.dptr = data;
		content.dsize = strlen(data) + 1;
		key.dptr = adata->message_id;
		key.dsize = strlen(adata->message_id) + 1;

		gdbm_store(dbf, key, content, GDBM_REPLACE);

		g_free(data);
		++count;
		gui_set_progress((gfloat) count / (gfloat) total);
		if (adata->threads)
			count = save_article_gslist(adata->threads, dbf, count, total);
	}
	return count;
}


article_data *
db_fetch_article (group_data *gdata, char *message_id)
{
	datum key, content;
	char **sd;
	article_data *adata = NULL;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_assert(gdata && message_id);

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if ((dbf = gdbm_open(path, 0, GDBM_READER | GDBM_FAST, 0600, 0)) ==
	    NULL) {
		g_print(_("We couldn't open the article database:\n%s\n"), path);
		return NULL;
	}
	key.dptr = message_id;
	key.dsize = strlen(message_id) + 1;

	content = gdbm_fetch (dbf, key);

	if (content.dptr) {
		adata = (article_data *) g_malloc0(sizeof(article_data));
		sd = g_strsplit(content.dptr, "\n", 9);

		adata->message_id = g_strdup(message_id);
		adata->date = strtol(sd[0], NULL, 10);
		adata->state = strtol(sd[1], NULL, 10);
		adata->linecount = strtol(sd[2], NULL, 10);
		adata->parts = strtol(sd[3], NULL, 10);
		adata->part = strtol(sd[4], NULL, 10);
		adata->number = sd[5];
		adata->subject = sd[6];
		adata->author = sd[7];
		adata->references = sd[8];
		adata->group = gdata;


		g_free (sd[0]);
		g_free (sd[1]);
		g_free (sd[2]);
		g_free (sd[3]);
		g_free (sd[4]);
		g_free (sd);
		g_free (content.dptr);
	}
	gdbm_close (dbf);
	return adata;
}


/*
 *--------------------------------------------------------------------
 * load all the articles from the specific group database
 *--------------------------------------------------------------------*/
/* FIXME: expiration isn't a real configuration option, a set default atm
 * TODO: use bz2 compression */
GSList *
db_load_articles (group_data *gdata)
{
	time_t current_time;
	int total = 0;
	GSList *alist = NULL;
	GSList *l = NULL;
	article_data *adata;
	char path[PATH_MAX];
	int expiration;			/* default set to 30 days */
	datum key, nextkey, content;
	GDBM_FILE dbf = NULL;
	char **sd;
	gfloat count = 0;

	expiration = gnome_config_get_int_with_default ("/Pan/Articles/Expiration=30", NULL);
	total = group_get_attrib_i (gdata, "Total");

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if ((dbf = gdbm_open(path, 0, GDBM_WRITER | GDBM_FAST, 0600, 0)) == NULL) {
		g_warning(_("Database didn't exist"));
		return NULL;
	}
	gui_set_status (_("Loading article headers"));
	time (&current_time);

	key = gdbm_firstkey(dbf);
	while (key.dptr) {
		nextkey = gdbm_nextkey (dbf, key);
		content = gdbm_fetch (dbf, key);
		if (content.dptr) {
			if ((current_time - atoi(content.dptr)) >= (86400 * expiration)) {
				l = g_slist_prepend (l, key.dptr);
			}
			else {
				adata = (article_data *) g_malloc0(sizeof(article_data));
				sd = g_strsplit (content.dptr, "\n", 9);

				adata->message_id = g_strdup(key.dptr);
				adata->date = strtol(sd[0], NULL, 10);
				adata->state = strtol(sd[1], NULL, 10);
				adata->linecount = strtol(sd[2], NULL, 10);
				adata->parts = strtol(sd[3], NULL, 10);
				adata->part = strtol(sd[4], NULL, 10);
				adata->number = sd[5];
				adata->subject = sd[6];
				adata->author = sd[7];
				adata->references = sd[8];
				adata->group = gdata;

				g_free (sd[0]);
				g_free (sd[1]);
				g_free (sd[2]);
				g_free (sd[3]);
				g_free (sd[4]);
				g_free (sd);

				alist = g_slist_prepend (alist, adata);
				count++;
			}
			g_free(content.dptr);
		}
		gui_set_progress ((gfloat) count/(gfloat) total);
		key = nextkey;
	}

	for (; l; l = l->next) {
		key.dptr = l->data;
		key.dsize = strlen(l->data) + 1;
		if (gdbm_delete(dbf, key) == -1)
			g_warning("article erase failed for message_id: %s \n", (char *)l->data);
		g_free (l->data);
	}

	gdbm_close (dbf);
	group_set_attrib_i (gdata, "Total", count);
	gui_set_progress (1);

	return alist;
}

void
db_save_articles (group_data *gdata, GSList *list, int total)
{
	int n = 0;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if (gdata->articlelist == list) /* If list is gdata->articlelist, we have them all */
		n = GDBM_NEWDB | GDBM_FAST;
	else
		n = GDBM_WRCREAT | GDBM_FAST; /* Else we have just a few to add */
	
	
	if ((dbf = gdbm_open(path, 0, n, 0600, 0)) == NULL) {
		g_print(_("We couldn't open the article database:\n%s\n"), path);
		return;
	}
	gui_set_progress (0);
	gui_set_status (_("Saving article headers"));
	n = save_article_gslist (list, dbf, 0, total);
	gui_set_progress (1);

	if (n > 0)
		gdata->flags |= GROUP_ONDISK;

	gdbm_close(dbf);
}


void
db_store_article (group_data *gdata, article_data *adata)
{
	datum key, content;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_return_if_fail (gdata && adata);

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if ((dbf = gdbm_open(path, 0, GDBM_WRITER | GDBM_FAST, 0600, 0)) ==
	    NULL) {
		g_warning(_("We couldn't open the article database:\n%s\n"), path);
		return;
	}
	content.dptr = g_strdup_printf ("%d\n%d\n%d\n%d\n%d\n%s\n%s\n%s\n%s",
					(int) adata->date, adata->state & ~STATE_NEW,
					adata->linecount, adata->parts,
					adata->part, adata->number,
					adata->subject, adata->author,
					adata->references);

	content.dsize = strlen (content.dptr) + 1;
	key.dptr = adata->message_id;
	key.dsize = strlen (adata->message_id) + 1;

	gdbm_store (dbf, key, content, GDBM_REPLACE);

	gdbm_close (dbf);
	g_free (content.dptr);
}


/* 
void group_index_erase(group_data * gdata)
{
char path[PATH_MAX];

g_snprintf(path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
Pan.active_server->name, gdata->name);

if (unlink(path) == -1)
pan_error_dialog(_("We couldn't modify the index for '%s'\n%s"),
gdata->name, g_strerror(errno));
}
*/

==> pan-0.6.2.tar.gz: pan-0.6.2/pan/db.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*
 TODO:
 [ ] transparent (de)compression of database (bzip2 preferably)
*/

#include <gdbm.h>
#include <errno.h>

#include "pan.h"


int
db_group_index_erase (group_data *gdata)
{
	char path[PATH_MAX];

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);
	if (unlink (path) < 0)
		return -1;
	else
		return 1;
}


void
db_article_erase (article_data *adata)
{
	datum key;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_return_if_fail(adata);

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, adata->group->name);

	if ((dbf = gdbm_open(path, 0, GDBM_WRITER | GDBM_FAST, 0600, 0)) == NULL) {
		g_warning(_("We couldn't open the article database:\n%s\n"), path);
		return;
	}

	key.dptr = adata->message_id;
	key.dsize = strlen(adata->message_id) + 1;

	if (gdbm_delete(dbf, key) == -1)
		g_warning ("article_db_erase(%s) failed\n", adata->message_id);
	gdbm_close(dbf);
}


int
group_index_exists (group_data *gdata)
{
	char path[PATH_MAX];

	g_snprintf(path, 256, "/%s/%s/%s.gdbm", data_dir,
		   Pan.active_server->name, gdata->name);

	if (g_file_exists(path))
		return TRUE;
	else
		return FALSE;
}


int
save_article_gslist (GSList *p, GDBM_FILE dbf, int count, int total)
{
	article_data *adata;
	time_t current_time;
	datum key, content;
	char *data = NULL;

	time(&current_time);

	for (; p; p = p->next) {
		adata = p->data;
		data = g_strdup_printf ("%d\n%d\n%d\n%d\n%d\n%s\n%s\n%s\n%s",
					(int) adata->date, adata->state & ~STATE_NEW,
					adata->linecount, adata->parts, adata->part,
					adata->number, adata->subject,
					adata->author, adata->references);

		content.dptr = data;
		content.dsize = strlen(data) + 1;
		key.dptr = adata->message_id;
		key.dsize = strlen(adata->message_id) + 1;

		gdbm_store(dbf, key, content, GDBM_REPLACE);

		g_free(data);
		++count;
		gui_set_progress((gfloat) count / (gfloat) total);
		if (adata->threads)
			count = save_article_gslist(adata->threads, dbf, count, total);
	}
	return count;
}


article_data *
db_fetch_article (group_data *gdata, char *message_id)
{
	datum key, content;
	char **sd;
	article_data *adata = NULL;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_assert(gdata && message_id);

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if ((dbf = gdbm_open(path, 0, GDBM_READER | GDBM_FAST, 0600, 0)) ==
	    NULL) {
		g_print(_("We couldn't open the article database:\n%s\n"), path);
		return NULL;
	}
	key.dptr = message_id;
	key.dsize = strlen(message_id) + 1;

	content = gdbm_fetch (dbf, key);

	if (content.dptr) {
		adata = (article_data *) g_malloc0(sizeof(article_data));
		sd = g_strsplit(content.dptr, "\n", 9);

		adata->message_id = g_strdup(message_id);
		adata->date = strtol(sd[0], NULL, 10);
		adata->state = strtol(sd[1], NULL, 10);
		adata->linecount = strtol(sd[2], NULL, 10);
		adata->parts = strtol(sd[3], NULL, 10);
		adata->part = strtol(sd[4], NULL, 10);
		adata->number = sd[5];
		adata->subject = sd[6];
		adata->author = sd[7];
		adata->references = sd[8];
		adata->group = gdata;


		g_free (sd[0]);
		g_free (sd[1]);
		g_free (sd[2]);
		g_free (sd[3]);
		g_free (sd[4]);
		g_free (sd);
		g_free (content.dptr);
	}
	gdbm_close (dbf);
	return adata;
}


/*
 *--------------------------------------------------------------------
 * load all the articles from the specific group database
 *--------------------------------------------------------------------*/
/* FIXME: expiration isn't a real configuration option, a set default atm
 * TODO: use bz2 compression */
GSList *
db_load_articles (group_data *gdata)
{
	time_t current_time;
	int total = 0;
	GSList *alist = NULL;
	GSList *expirelist = NULL;
	article_data *adata;
	char path[PATH_MAX];
	int expiration;			/* default set to 30 days */
	datum key, nextkey, content;
	GDBM_FILE dbf = NULL;
	char **sd;
	gfloat count = 0;

	expiration = gnome_config_get_int_with_default ("/Pan/Articles/Expiration=30", NULL);
	total = group_get_attrib_i (gdata, "Total");

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if ((dbf = gdbm_open(path, 0, GDBM_WRITER | GDBM_FAST, 0600, 0)) == NULL) {
		g_warning(_("Database didn't exist"));
		return NULL;
	}
	gui_set_status (_("Loading article headers"));
	time (&current_time);

	key = gdbm_firstkey(dbf);
	while (key.dptr) {
		nextkey = gdbm_nextkey (dbf, key);
		content = gdbm_fetch (dbf, key);
		if (content.dptr) {
			if ((current_time - atoi(content.dptr)) >= (86400 * expiration)) {
				expirelist = g_slist_prepend (expirelist, key.dptr);
			}
			else {
				adata = (article_data *) g_malloc0(sizeof(article_data));
				sd = g_strsplit (content.dptr, "\n", 9);

				adata->message_id = g_strdup (key.dptr);
				adata->date = strtol (sd[0], NULL, 10);
				adata->state = strtol (sd[1], NULL, 10);
				adata->linecount = strtol (sd[2], NULL, 10);
				adata->parts = strtol (sd[3], NULL, 10);
				adata->part = strtol (sd[4], NULL, 10);
				adata->number = g_strdup (sd[5]);
				adata->subject = g_strdup (sd[6]);
				adata->author = g_strdup (sd[7]);
				adata->references = g_strdup (sd[8]);
				adata->group = gdata;

//				g_free (sd[0]);
//				g_free (sd[1]);
//				g_free (sd[2]);
//				g_free (sd[3]);
//				g_free (sd[4]);
//				g_free (sd);
				g_strfreev (sd);

				alist = g_slist_prepend (alist, adata);
				count++;
			}
			g_free (content.dptr);
		}
		gui_set_progress ((gfloat) count/(gfloat) total);
		free (key.dptr);
		key = nextkey;
	}


	for (; expirelist; expirelist = expirelist->next) {
		key.dptr = expirelist->data;
		key.dsize = strlen (expirelist->data) + 1;
		if (expirelist->data && gdbm_delete (dbf, key) == -1)
			g_warning ("article erase failed for message_id: %s \n", (char *)expirelist->data);
//		g_free (expirelist->data);
	}
	g_slist_free (expirelist);

	gdbm_close (dbf);
	group_set_attrib_i (gdata, "Total", count);
	gui_set_progress (1);

	return alist;
}

void
db_save_articles (group_data *gdata, GSList *list, int total)
{
	int n = 0;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if (gdata->articlelist == list) /* If list is gdata->articlelist, we have them all */
		n = GDBM_NEWDB | GDBM_FAST;
	else
		n = GDBM_WRCREAT | GDBM_FAST; /* Else we have just a few to add */
	
	
	if ((dbf = gdbm_open(path, 0, n, 0600, 0)) == NULL) {
		g_print(_("We couldn't open the article database:\n%s\n"), path);
		return;
	}
	gui_set_progress (0);
	gui_set_status (_("Saving article headers"));
	n = save_article_gslist (list, dbf, 0, total);
	gui_set_progress (1);

	if (n > 0)
		gdata->flags |= GROUP_ONDISK;

	gdbm_close(dbf);
}


void
db_store_article (group_data *gdata, article_data *adata)
{
	datum key, content;
	char path[PATH_MAX];
	GDBM_FILE dbf = NULL;

	g_return_if_fail (gdata && adata);

	g_snprintf (path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
		    Pan.active_server->name, gdata->name);

	if ((dbf = gdbm_open(path, 0, GDBM_WRITER | GDBM_FAST, 0600, 0)) ==
	    NULL) {
		g_warning(_("We couldn't open the article database:\n%s\n"), path);
		return;
	}
	content.dptr = g_strdup_printf ("%d\n%d\n%d\n%d\n%d\n%s\n%s\n%s\n%s",
					(int) adata->date, adata->state & ~STATE_NEW,
					adata->linecount, adata->parts,
					adata->part, adata->number,
					adata->subject, adata->author,
					adata->references);

	content.dsize = strlen (content.dptr) + 1;
	key.dptr = adata->message_id;
	key.dsize = strlen (adata->message_id) + 1;

	gdbm_store (dbf, key, content, GDBM_REPLACE);

	gdbm_close (dbf);
	g_free (content.dptr);
}


/* 
void group_index_erase(group_data * gdata)
{
char path[PATH_MAX];

g_snprintf(path, PATH_MAX, "/%s/%s/%s.gdbm", data_dir,
Pan.active_server->name, gdata->name);

if (unlink(path) == -1)
pan_error_dialog(_("We couldn't modify the index for '%s'\n%s"),
gdata->name, g_strerror(errno));
}
*/
